;; Copyright (c) 2021, rrc-open-firmware FM10K-Documentation Contributors
;;
;; Redistribution and use in source and binary forms, with or without
;; modification, are permitted provided that the following conditions are met:
;;
;;     * Redistributions of source code must retain the above copyright notice,
;;       this list of conditions and the following disclaimer.
;;     * Redistributions in binary form must reproduce the above copyright
;;       notice, this list of conditions and the following disclaimer in the
;;       documentation and/or other materials provided with the distribution.
;;     * Neither the name of the copyright holder nor the names of its contributors
;;       may be used to endorse or promote products derived from this software
;;       without specific prior written permission.
;;
;; THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
;; AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
;; DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
;; FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
;; DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
;; SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
;; CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
;; OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
;; OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

interrupt_handler:
    ; TODO: what happens if we receive an interrupt while on another BSM method? can it happen?
    MOV api_BSM_STATUS, api_BSM_STATUS.ISR_Starting
    WRITE rrcc_STACK_POINTER, rrcc_STACK_START                  ; Reset Stack pointer to its starting position
    ;;WRITE R0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0    ; Zero R0 - RF
    INIT R0, 16 ; Zero R0 - RF

    MOV global_PEP_interrupts_counter, 0

    INIT platform_PEP0_COUNTER, 9 ; Zero platform_PEP0_COUNTER - platform_PEP8_COUNTER
    INIT platform_PEP0_COUNTER_2, 9 ; Zero platform_PEP0_COUNTER_2 - platform_PEP8_COUNTER_2

    WRITE api_RE_RESET_MASK_STATUS_1, 0x00000000, 0x00000000    ; 64-bit write

    .takeSoftResetLock: CALL @lock_soft_reset_take
    BNE RRET, 1, 0x1, @.takeSoftResetLock                       ; Failed to take lock, keep retrying in busy loop

    CALL @interrupt_handle_each_source
    ; TODO this

    END


lock_soft_reset_take:
    PNE api_SOFT_RESET_LOCK_STATE, 0, 0x3, 20000, 50, @.failTakeLock
        SET api_SOFT_RESET_LOCK_STATE, platform_SOFT_RESET_LOCK_NVM, 0x3
        BNE api_SOFT_RESET_LOCK_STATE, platform_SOFT_RESET_LOCK_NVM, 0x3, @.retryLock
            RET 1 ; Success
        .retryLock:
        PNE api_SOFT_RESET_LOCK_STATE, 0, 0x3, 20000, 50, @.failTakeLock
        SET api_SOFT_RESET_LOCK_STATE, platform_SOFT_RESET_LOCK_NVM, 0x3
        BNE api_SOFT_RESET_LOCK_STATE, platform_SOFT_RESET_LOCK_NVM, 0x3, @.failTakeLock
            RET 1 ; Success
    .failTakeLock:
    SET api_SW_LOCK_ERR_STATUS, 0x00000100, 0x00000100
    BEQ api_SW_LOCK_ERR_STATUS, 0xfe00, 0xfe00, @.returnFail
        ADD api_SW_LOCK_ERR_STATUS, api_SW_LOCK_ERR_STATUS, 0x200
        RET 0 ; Failure
    .returnFail: SET api_SW_LOCK_ERR_STATUS, 0x0000, 0xfe00
    ADD api_SW_LOCK_ERR_STATUS, api_SW_LOCK_ERR_STATUS, 0x200
    RET 0 ; Failure

lock_soft_reset_release:
    BNE api_SOFT_RESET_LOCK_STATE, platform_SOFT_RESET_LOCK_NVM, 0x3, @.return  ; If we hold the lock
        SET api_SOFT_RESET_LOCK_STATE, platform_SOFT_RESET_LOCK_FREE, 0x3       ; Release lock
    .return:
    RET

interrupt_handle_each_source:
    MOV global_PEP_NUMBER, 0

    .handleHostPEPsLoop:
    MOV R1, 1
    SHL R0, R1, global_PEP_NUMBER
    AND R1, GLOBAL_INTERRUPT_DETECT_0, R0 ; Get specific INTERRUPTS_0.PCIE_BSM[0-8] field
    BEQ R1, 0, 0xFFFFFFFF, @.skipPEP ; Skip if PEP number does not need handling
        SHL R1, R0, 16 ; Set proper flags on status
        OR R1, api_BSM_STATUS, api_BSM_STATUS.Interrupt_Host
        MOV api_BSM_STATUS, R1
        MOV R1, platform_PEP0_COUNTER &
        ADD global_PEP_SCRATCH_ADDR, R1, global_PEP_NUMBER ; Set global_PEP_SCRATCH_ADDR according to current PEP
        FASTCALL @interrupt_set_clocks_once_for_PEP, global_PEP_NUMBER
        MOV BSM_ADDR_OFFSET_1, global_PEP_SCRATCH_ADDR
        BNE~1 std_THIS, 0, 0xFFFFFFFF, @.donePEP ; If interrupt not handled, handle it (?)
            .setupPEP:
            FASTCALL @std_multiply, PCIE_PF_OFFSET, global_PEP_NUMBER
            MOV BSM_ADDR_OFFSET_1, RRET ; pepNumber * PCIE_PF_OFFSET
            BNE~1 PCIE_IP, PCIE_IP.OutOfReset, PCIE_IP.OutOfReset, @.skipPEPHandling ; If PCIE_IP not OutOfReset
            SET api_BSM_STATUS, 0x01000000, 0x01000000 ; Interrupt Processing, OutOfReset
            BEQ~1 PCIE_IP, PCIE_IP.PFLR, PCIE_IP.PFLR, @.skipPEPHandling ; If PCIE_IP has PFLR
            BEQ~1 PCIE_IP, PCIE_IP.DataPathReset, PCIE_IP.DataPathReset, @.skipPEPHandling ; If PCIE_IP has DataPathReset
                .handlePEP:
                MOV BSM_ADDR_OFFSET_1, global_PEP_SCRATCH_ADDR
                MOV~10 std_THIS, 0x01 ; Set PEP scratch status 0x1
                MOV~01 global_PEP_interrupts_counter, std_THIS +9
                ADD global_PEP_interrupts_counter, global_PEP_interrupts_counter, 1
                FASTCALL @interrupt_handle_PEP, global_PEP_NUMBER
                MOV BSM_ADDR_OFFSET_1, global_PEP_SCRATCH_ADDR
                MOV~10 std_THIS, 0x02 ; Set PEP scratch status 0x2
                MOV R1, 1
                SHL R1, R1, global_PEP_NUMBER
                OR api_RE_RESET_MASK_STATUS_1, api_RE_RESET_MASK_STATUS_1, R1
            .skipPEPHandling:
            MOV BSM_ADDR_OFFSET_1, global_PEP_SCRATCH_ADDR
            MOV~10 std_THIS, 0x10 ; Set PEP scratch status 0x10
        MOV BSM_ADDR_OFFSET_1, global_PEP_NUMBER
        .donePEP: MOV~10 api_PCIE_ISR_STATUS_0, api_BSM_STATUS ; Set proper ISR status field based on offset

    .skipPEP:
    ADD global_PEP_NUMBER, global_PEP_NUMBER, 1
    BNE global_PEP_NUMBER, 9, 0xFFFFFFFF, @.handleHostPEPsLoop ; Until PEP != 9 ( < 9, 0...8)

    BEQ api_RE_RESET_MASK_STATUS_1, 0, 0x000001ff, @.ret ; No additional masks to handle reset, skip loop

    WAIT 220000

    MOV global_PEP_NUMBER, 0

    .handleHostResetMaskLoop:
    MOV R1, 1
    SHL R0, R1, global_PEP_NUMBER
    AND R1, api_RE_RESET_MASK_STATUS_1, R0 ; Get specific api_RE_RESET_MASK_STATUS_1 field
    BEQ R1, 0, 0xFFFFFFFF, @.skipHostReset ; Skip if PEP number does not need handling
        ADD BSM_ADDR_OFFSET_1, global_PEP_NUMBER, platform_PEP0_COUNTER & ; Set BSM_ADDR_OFFSET_1 according to current PEP scratch
        ADD~1 std_THIS, std_THIS, 1
        BNE std_THIS, 6, 0xf, @.skipUnknown
            MOV~01 R0, std_THIS +9 ; Use second counter which is +9 off
            SUB unknown_PEP_counter, global_PEP_interrupts_counter, R0 ; Use platform_PEP0_COUNTER_2
            BEQ unknown_PEP_counter, 0, 0xfffffffc, @.skipUnknown
            ADD~1 std_THIS, std_THIS, 1
        .skipUnknown:
        BNE platform_PEP0_COUNTER, 6, 0xf, @.skipHostReset
        SET api_RE_RESET_MASK_STATUS_1, 0, R1
    .skipHostReset:
    ADD global_PEP_NUMBER, global_PEP_NUMBER, 1
    BNE global_PEP_NUMBER, 9, 0xFFFFFFFF, @.handleHostResetMaskLoop ; Until PEP != 9 ( < 9, 0...8)

    .ret: RET


; FASTCALL void(uint pepNumber)
interrupt_handle_PEP:
    MOV R0, P0
    FASTCALL @std_multiply, PCIE_PF_OFFSET, R0
    MOV BSM_ADDR_OFFSET_1, RRET ; pepNumber * PCIE_PF_OFFSET
    MOV~10 PCIE_IP, PCIE_IP.OutOfReset ; Clears OutOfReset bit on PCIE_IP (CW1)
    FASTCALL @pcie_PEP_reset_enable, R0
    FASTCALL @interrupt_PEP_SerDes_Interrupt, R0, 1

    BEQ api_PCIE_SBUS_LOCK_STATE, platform_PCIE_SBUS_LOCK_NVM, 0x3, @.lockTaken ; Check if we have lock
        .takeLock: CALL @lock_PCIE_SBUS_take
        BNE RRET, 1, 0x1, @.takeLock ; We failed to take the lock
    .lockTaken: FASTCALL @interrupt_handle_PEP_SerDes, R0
    FASTCALL @pcie_PEP_reset_disable, R0
    FASTCALL @interrupt_PEP_SerDes_Interrupt, R0, 0
    RET


; FASTCALL void(uint pepNumber), uses R14 and R15
interrupt_handle_PEP_SerDes:
    BNE P0, 8, 0xFFFFFFFF, @.notPEP8
        MOV R14, 0x42
        MOV R15, 0x43
        JUMP @.exec
    .notPEP8:
        ADD R14, P0, 0xe0 ; PEP_DEV_0 = 0xe0 + pepNumber
        ADD R15, P0, 0xd0 ; PEP_DEV_1 = 0xd0 + pepNumber
    .exec:

    FASTCALL @execute_SBus_PCIE_Command, 0x07, R14, 0, 0x00000011
    BNE RRET, 1, 0x1, @.failCommand ; Command failed
    WAIT 100

    FASTCALL @execute_SBus_PCIE_Command, 0x07, R14, 0, 0x00000010
    BNE RRET, 1, 0x1, @.failCommand ; Command failed

    FASTCALL @execute_SBus_PCIE_Command, 0x07, R14, 0, 0x00000002
    BNE RRET, 1, 0x1, @.failCommand ; Command failed

    FASTCALL @execute_SBus_PCIE_Command, 0x03, R14, 0, 0x00060050
    BNE RRET, 1, 0x1, @.failCommand ; Command failed

    FASTCALL @execute_SBus_PCIE_Command, 0x03, R14, 0, 0x00051032
    BNE RRET, 1, 0x1, @.failCommand ; Command failed

    FASTCALL @execute_SBus_PCIE_Command, 0x03, R14, 0, 0x0026000e
    BNE RRET, 1, 0x1, @.failCommand ; Command failed

    FASTCALL @execute_SBus_PCIE_Command, 0x03, R14, 0, 0x00260102
    BNE RRET, 1, 0x1, @.failCommand ; Command failed

    FASTCALL @execute_SBus_PCIE_Command, 0x03, R14, 0, 0x00260238
    BNE RRET, 1, 0x1, @.failCommand ; Command failed

    FASTCALL @execute_SBus_PCIE_Command, 0x03, R14, 0, 0x00265202
    BNE RRET, 1, 0x1, @.failCommand ; Command failed

    FASTCALL @execute_SBus_PCIE_Command, 0x03, R14, 0, 0x00265b01
    BNE RRET, 1, 0x1, @.failCommand ; Command failed

    FASTCALL @execute_SBus_PCIE_Command, 0x03, R14, 0, 0x002b0000
    BNE RRET, 1, 0x1, @.failCommand ; Command failed

    FASTCALL @execute_SBus_PCIE_Command, 0x03, R14, 0, 0x00184240
    BNE RRET, 1, 0x1, @.failCommand ; Command failed

    FASTCALL @execute_SBus_PCIE_Command, 0x03, R14, 0, 0x00190084
    BNE RRET, 1, 0x1, @.failCommand ; Command failed

    FASTCALL @execute_SBus_PCIE_Command, 0x03, R15, 0, 0x00022132
    BNE RRET, 1, 0x1, @.failCommand ; Command failed

    FASTCALL @execute_SBus_PCIE_Command, 0x0f, R15, 0, 0x22041604
    BNE RRET, 1, 0x1, @.failCommand ; Command failed

    FASTCALL @execute_SBus_PCIE_Command, 0x12, R15, 0, 0x00009604
    BNE RRET, 1, 0x1, @.failCommand ; Command failed

    JUMP @.ret

    .failCommand:
    MOV api_SERDES_OOR_STATUS_PASS_1, 0x08000000 ; Write fail
    OR api_SERDES_OOR_STATUS_PASS_1, api_SERDES_OOR_STATUS_PASS_1, R14
    .ret: RET

; FASTCALL void(uint pepNumber, uint value), uses R14, R15
interrupt_PEP_SerDes_Interrupt:
    MOV R14, P0
    MOV R15, P1
    FASTCALL @std_multiply, PCIE_PF_OFFSET, R14
    MOV BSM_ADDR_OFFSET_1, RRET ; pepNumber * PCIE_PF_OFFSET

    MOV~10 PCIE_SERDES_CTRL_0a, R15 ; PCIE_SERDES_CTRL_0a
    MOV~10 PCIE_SERDES_CTRL_1a, R15 ; PCIE_SERDES_CTRL_1a
    MOV~10 PCIE_SERDES_CTRL_2a, R15 ; PCIE_SERDES_CTRL_2a
    MOV~10 PCIE_SERDES_CTRL_3a, R15 ; PCIE_SERDES_CTRL_3a

    FASTCALL @pcie_get_PEP_mode R14
    BEQ RRET, 0x1, 0x1, @.skip_pep_1x8 ; Skip if PEP Mode is 2x4
        MOV~10 PCIE_SERDES_CTRL_4a, R15 ; PCIE_SERDES_CTRL_4a
        MOV~10 PCIE_SERDES_CTRL_5a, R15 ; PCIE_SERDES_CTRL_5a
        MOV~10 PCIE_SERDES_CTRL_6a, R15 ; PCIE_SERDES_CTRL_6a
        MOV~10 PCIE_SERDES_CTRL_7a, R15 ; PCIE_SERDES_CTRL_7a

    .skip_pep_1x8: RET
    RET

; FASTCALL void(uint pepNumber)
; This could be made faster with custom RETURN. Better not to!
interrupt_set_clocks_once_for_PEP:
    MOV R0, P0
    BNE R0, 0, 0xFFFFFFFF, @.skipPEP0
        CALL @interrupt_set_clocks_once_PEP0
        JUMP @.ret
    .skipPEP0:
    BNE R0, 1, 0xFFFFFFFF, @.skipPEP1
        CALL @interrupt_set_clocks_once_PEP1
        JUMP @.ret
    .skipPEP1:
    BNE R0, 2, 0xFFFFFFFF, @.skipPEP2
        CALL @interrupt_set_clocks_once_PEP2
        JUMP @.ret
    .skipPEP2:
    BNE R0, 3, 0xFFFFFFFF, @.skipPEP3
        CALL @interrupt_set_clocks_once_PEP3
        JUMP @.ret
    .skipPEP3:
    BNE R0, 4, 0xFFFFFFFF, @.skipPEP4
        CALL @interrupt_set_clocks_once_PEP4
        JUMP @.ret
    .skipPEP4:
    BNE R0, 5, 0xFFFFFFFF, @.skipPEP5
        CALL @interrupt_set_clocks_once_PEP5
        JUMP @.ret
    .skipPEP5:
    BNE R0, 6, 0xFFFFFFFF, @.skipPEP6
        CALL @interrupt_set_clocks_once_PEP6
        JUMP @.ret
    .skipPEP6:
    BNE R0, 7, 0xFFFFFFFF, @.skipPEP7
        CALL @interrupt_set_clocks_once_PEP7
        JUMP @.ret
    .skipPEP7:
    BNE R0, 8, 0xFFFFFFFF, @.ret
        CALL @interrupt_set_clocks_once_PEP8
    .ret: RET

interrupt_set_clocks_once_PEP0:
    BEQ CHIP_VERSION, 0x00, CHIP_VERSION.Version, @.skipReset
        BEQ PCIE_CLK_CTRL, 0, PCIE_CLK_CTRL.Mode_0, @.skipReset ; If Mode for PEP group = PCIE_REFCLK
            BNE PCIE_CLK_STAT, 0, PCIE_CLK_STAT.RefclkSel_0, @.skipReset ; If RefclkSel != 0
                AND platform_PEP0_PCIE_CLK_CTRL, PCIE_CLK_CTRL, PCIE_CLK_CTRL.Mode_0
                SET SOFT_RESET, SOFT_RESET.PCIeReset_0, SOFT_RESET.PCIeReset_0 ; Set PCIeReset for PEP = 1
                SET PCIE_CLK_CTRL, 0, PCIE_CLK_CTRL.Mode_0; Set Mode for PEP group = PCIE_REFCLK
                SET SOFT_RESET, 0, SOFT_RESET.PCIeReset_0 ; Set PCIeReset for PEP = 0
    .skipReset: SET SOFT_RESET, SOFT_RESET.PCIeActive_0, SOFT_RESET.PCIeActive_0 ; Set PCIeActive for PEP = 1
    RET

interrupt_set_clocks_once_PEP1:
    BEQ CHIP_VERSION, 0x00, CHIP_VERSION.Version, @.skipReset
        BEQ PCIE_CLK_CTRL, 0, PCIE_CLK_CTRL.Mode_0, @.skipReset ; If Mode for PEP group = PCIE_REFCLK
            BNE PCIE_CLK_STAT, 0, PCIE_CLK_STAT.RefclkSel_0, @.skipReset ; If RefclkSel != 0
                AND platform_PEP1_PCIE_CLK_CTRL, PCIE_CLK_CTRL, PCIE_CLK_CTRL.Mode_0
                SET SOFT_RESET, SOFT_RESET.PCIeReset_1, SOFT_RESET.PCIeReset_1 ; Set PCIeReset for PEP = 1
                SET PCIE_CLK_CTRL, 0, PCIE_CLK_CTRL.Mode_0; Set Mode for PEP group = PCIE_REFCLK
                SET SOFT_RESET, 0, SOFT_RESET.PCIeReset_1 ; Set PCIeReset for PEP = 0
    .skipReset: SET SOFT_RESET, SOFT_RESET.PCIeActive_1, SOFT_RESET.PCIeActive_1 ; Set PCIeActive for PEP = 1
    RET

interrupt_set_clocks_once_PEP2:
    BEQ CHIP_VERSION, 0x00, CHIP_VERSION.Version, @.skipReset
        BEQ PCIE_CLK_CTRL, 0, PCIE_CLK_CTRL.Mode_2, @.skipReset ; If Mode for PEP group = PCIE_REFCLK
            BNE PCIE_CLK_STAT, 0, PCIE_CLK_STAT.RefclkSel_2, @.skipReset ; If RefclkSel != 0
                AND platform_PEP2_PCIE_CLK_CTRL, PCIE_CLK_CTRL, PCIE_CLK_CTRL.Mode_2
                SET SOFT_RESET, SOFT_RESET.PCIeReset_2, SOFT_RESET.PCIeReset_2 ; Set PCIeReset for PEP = 1
                SET PCIE_CLK_CTRL, 0, PCIE_CLK_CTRL.Mode_2; Set Mode for PEP group = PCIE_REFCLK
                SET SOFT_RESET, 0, SOFT_RESET.PCIeReset_2 ; Set PCIeReset for PEP = 0
    .skipReset: SET SOFT_RESET, SOFT_RESET.PCIeActive_2, SOFT_RESET.PCIeActive_2 ; Set PCIeActive for PEP = 1
    RET
    
interrupt_set_clocks_once_PEP3:
    BEQ CHIP_VERSION, 0x00, CHIP_VERSION.Version, @.skipReset
        BEQ PCIE_CLK_CTRL, 0, PCIE_CLK_CTRL.Mode_2, @.skipReset ; If Mode for PEP group = PCIE_REFCLK
            BNE PCIE_CLK_STAT, 0, PCIE_CLK_STAT.RefclkSel_2, @.skipReset ; If RefclkSel != 0
                AND platform_PEP3_PCIE_CLK_CTRL, PCIE_CLK_CTRL, PCIE_CLK_CTRL.Mode_2
                SET SOFT_RESET, SOFT_RESET.PCIeReset_3, SOFT_RESET.PCIeReset_3 ; Set PCIeReset for PEP = 1
                SET PCIE_CLK_CTRL, 0, PCIE_CLK_CTRL.Mode_2; Set Mode for PEP group = PCIE_REFCLK
                SET SOFT_RESET, 0, SOFT_RESET.PCIeReset_3 ; Set PCIeReset for PEP = 0
    .skipReset: SET SOFT_RESET, SOFT_RESET.PCIeActive_3, SOFT_RESET.PCIeActive_3 ; Set PCIeActive for PEP = 1
    RET

interrupt_set_clocks_once_PEP4:
    BEQ CHIP_VERSION, 0x00, CHIP_VERSION.Version, @.skipReset
        BEQ PCIE_CLK_CTRL, 0, PCIE_CLK_CTRL.Mode_4, @.skipReset ; If Mode for PEP group = PCIE_REFCLK
            BNE PCIE_CLK_STAT, 0, PCIE_CLK_STAT.RefclkSel_4, @.skipReset ; If RefclkSel != 0
                AND platform_PEP4_PCIE_CLK_CTRL, PCIE_CLK_CTRL, PCIE_CLK_CTRL.Mode_4
                SET SOFT_RESET, SOFT_RESET.PCIeReset_4, SOFT_RESET.PCIeReset_4 ; Set PCIeReset for PEP = 1
                SET PCIE_CLK_CTRL, 0, PCIE_CLK_CTRL.Mode_4; Set Mode for PEP group = PCIE_REFCLK
                SET SOFT_RESET, 0, SOFT_RESET.PCIeReset_4 ; Set PCIeReset for PEP = 0
    .skipReset: SET SOFT_RESET, SOFT_RESET.PCIeActive_4, SOFT_RESET.PCIeActive_4 ; Set PCIeActive for PEP = 1
    RET

interrupt_set_clocks_once_PEP5:
    BEQ CHIP_VERSION, 0x00, CHIP_VERSION.Version, @.skipReset
        BEQ PCIE_CLK_CTRL, 0, PCIE_CLK_CTRL.Mode_4, @.skipReset ; If Mode for PEP group = PCIE_REFCLK
            BNE PCIE_CLK_STAT, 0, PCIE_CLK_STAT.RefclkSel_4, @.skipReset ; If RefclkSel != 0
                AND platform_PEP5_PCIE_CLK_CTRL, PCIE_CLK_CTRL, PCIE_CLK_CTRL.Mode_4
                SET SOFT_RESET, SOFT_RESET.PCIeReset_5, SOFT_RESET.PCIeReset_5 ; Set PCIeReset for PEP = 1
                SET PCIE_CLK_CTRL, 0, PCIE_CLK_CTRL.Mode_4; Set Mode for PEP group = PCIE_REFCLK
                SET SOFT_RESET, 0, SOFT_RESET.PCIeReset_5 ; Set PCIeReset for PEP = 0
    .skipReset: SET SOFT_RESET, SOFT_RESET.PCIeActive_5, SOFT_RESET.PCIeActive_5 ; Set PCIeActive for PEP = 1
    RET
    
interrupt_set_clocks_once_PEP6:
    BEQ CHIP_VERSION, 0x00, CHIP_VERSION.Version, @.skipReset
        BEQ PCIE_CLK_CTRL, 0, PCIE_CLK_CTRL.Mode_6, @.skipReset ; If Mode for PEP group = PCIE_REFCLK
            BNE PCIE_CLK_STAT, 0, PCIE_CLK_STAT.RefclkSel_6, @.skipReset ; If RefclkSel != 0
                AND platform_PEP6_PCIE_CLK_CTRL, PCIE_CLK_CTRL, PCIE_CLK_CTRL.Mode_6
                SET SOFT_RESET, SOFT_RESET.PCIeReset_6, SOFT_RESET.PCIeReset_6 ; Set PCIeReset for PEP = 1
                SET PCIE_CLK_CTRL, 0, PCIE_CLK_CTRL.Mode_6; Set Mode for PEP group = PCIE_REFCLK
                SET SOFT_RESET, 0, SOFT_RESET.PCIeReset_6 ; Set PCIeReset for PEP = 0
    .skipReset: SET SOFT_RESET, SOFT_RESET.PCIeActive_6, SOFT_RESET.PCIeActive_6 ; Set PCIeActive for PEP = 1
    RET
    
interrupt_set_clocks_once_PEP7:
    BEQ CHIP_VERSION, 0x00, CHIP_VERSION.Version, @.skipReset
        BEQ PCIE_CLK_CTRL, 0, PCIE_CLK_CTRL.Mode_6, @.skipReset ; If Mode for PEP group = PCIE_REFCLK
            BNE PCIE_CLK_STAT, 0, PCIE_CLK_STAT.RefclkSel_6, @.skipReset ; If RefclkSel != 0
                AND platform_PEP7_PCIE_CLK_CTRL, PCIE_CLK_CTRL, PCIE_CLK_CTRL.Mode_6
                SET SOFT_RESET, SOFT_RESET.PCIeReset_7, SOFT_RESET.PCIeReset_7 ; Set PCIeReset for PEP = 1
                SET PCIE_CLK_CTRL, 0, PCIE_CLK_CTRL.Mode_6; Set Mode for PEP group = PCIE_REFCLK
                SET SOFT_RESET, 0, SOFT_RESET.PCIeReset_7 ; Set PCIeReset for PEP = 0
    .skipReset: SET SOFT_RESET, SOFT_RESET.PCIeActive_7, SOFT_RESET.PCIeActive_7 ; Set PCIeActive for PEP = 1
    RET


interrupt_set_clocks_once_PEP8:
    SET SOFT_RESET, SOFT_RESET.PCIeActive_8, SOFT_RESET.PCIeActive_8 ; Set PEP8 active via SOFT_RESET.PCIeActive_8
    RET